@@ -1,14 +1,16 @@
 #! /usr/bin/python
-import rowingdata
+from __future__ import absolute_import
+from __future__ import print_function
+from . import rowingdata
 from sys import argv
 
 def main():
     readFile=argv[1]
 
     try:
-	rowerFile=argv[2]
+        rowerFile=argv[2]
     except IndexError:
-	rowerFile="defaultrower.txt"
+        rowerFile="defaultrower.txt"
 
     rower=rowingdata.getrower(rowerFile)
 
@@ -18,7 +20,7 @@ def main():
     rp.write_csv(csvoutput)
 
     res=rowingdata.rowingdata(csvoutput,rowtype="Indoor Rower",
-				rower=rower)
+                                rower=rower)
 
 
     tcxoutput=readFile+"_o.tcx"
@@ -27,4 +29,4 @@ def main():
 
 
 
-    print("done "+readFile)
+    print(("done "+readFile))
